import 'package:flutter/widgets.dart';
import 'package:flutter/foundation.dart';
import 'package:bloc_manager_cluster/bloc_manager_cluster.dart';

class BlocManagerClusterProvider extends InheritedWidget {
  final BlocManagerCluster cluster;

  BlocManagerClusterProvider({
    Key key,
    Widget child,
    @required List<BlocManager> managers,
  })  : cluster = BlocManagerCluster(managers: managers),
        super(key: key, child: child);

  static BlocManagerCluster of(BuildContext context) {
    return context
        .dependOnInheritedWidgetOfExactType<BlocManagerClusterProvider>()
        .cluster;
  }

  @override
  bool updateShouldNotify(InheritedWidget oldWidget) {
    return false;
  }
}

class BlocManagerCluster {
  final Map<Type, BlocManager> _managers = {};

  BlocManagerCluster({
    @required List<BlocManager> managers,
  }) {
    managers.forEach(_register);
  }

  void _register(BlocManager manager) {
    manager.cluster = this;
    _managers[manager.runtimeType] = manager;
  }

  B manager<B extends BlocManager>() => _managers[B];

  void allotAction(BlocAction action) =>
      _managers.values.forEach((m) => m.receiveAction(action));
}
